package main

import (
	"crypto/x509"
	"encoding/hex"
	"encoding/pem"
	"fmt"

	crypto2 "github.com/libp2p/go-libp2p-core/crypto"
	"github.com/libp2p/go-libp2p-core/peer"

	"github.com/link33/sidecar/tool"
	"github.com/meshplus/bitxhub-kit/crypto"
	"github.com/meshplus/bitxhub-kit/crypto/asym"
	"github.com/urfave/cli"
)

var toolCMD = cli.Command{
	Name:  "tool",
	Usage: "generate tool for node",
	Subcommands: []cli.Command{
		{
			Name:  "p2p",
			Usage: "about p2p information",
			Action: func(ctx *cli.Context) error {
				// 指定类型ECDSA_P256，随机产生p2p私钥
				p2pPrivateKey, err := asym.GenerateKeyPair(crypto.ECDSA_P256)
				tool.Asset(err)

				// 编码16
				priKeyEncode, err := p2pPrivateKey.Bytes()
				tool.Asset(err)

				// 产生x509 ecdsa.PrivateKey 类型私钥
				stdKey, err := x509.ParseECPrivateKey(priKeyEncode)
				tool.Asset(err)

				pemEncodePrivrateData := pem.EncodeToMemory(&pem.Block{Type: "EC PRIVATE KEY", Bytes: priKeyEncode})
				fmt.Println("--------p2p node peer PrivateKey ----------begin\n")
				fmt.Println(string(pemEncodePrivrateData))
				fmt.Println("--------p2p node peer PrivateKey----------end\n")

				// 类型转换
				k, err := asym.PrivateKeyFromStdKey(stdKey)
				tool.Asset(err)
				sdKey, err := asym.PrivKeyToStdKey(k)
				tool.Asset(err)

				_, pk, err := crypto2.KeyPairFromStdKey(&sdKey)
				tool.Asset(err)
				id, err := peer.IDFromPublicKey(pk)
				tool.Asset(err)
				fmt.Println("--------p2p node ID----------begin")
				fmt.Println(id.Pretty())
				fmt.Println("--------p2p node ID----------end")
				return nil
			},
		},
		{
			Name:  "appchain",
			Usage: "about appchain information",
			Action: func(ctx *cli.Context) error {
				privKey, err := asym.GenerateKeyPair(crypto.Secp256k1)
				tool.Asset(err)

				dataPrivKey, err := privKey.Bytes()
				tool.Asset(err)

				fmt.Println("--------user PrivateKey----------begin")
				fmt.Println(hex.EncodeToString(dataPrivKey))
				fmt.Println("--------user PrivateKey----------end \n")

				publicKey, err := privKey.PublicKey().Bytes()
				tool.Asset(err)

				fmt.Println("--------user PublicKey----------begin")
				fmt.Println(hex.EncodeToString(publicKey))
				fmt.Println("--------user PublicKey----------end \n")

				address, err := privKey.PublicKey().Address()
				tool.Asset(err)

				fmt.Println("--------user Address----------begin")
				fmt.Println(address.String())
				fmt.Println("--------user Address----------end \n")

				keyStore, err := asym.GenKeyStore(privKey, "link33")
				data, err := keyStore.Pretty()
				tool.Asset(err)
				fmt.Println("--------user keyStore----------begin ")
				fmt.Println(data)
				fmt.Println("--------user keyStore----------end \n")

				return nil
			},
		},
	},
}
